import requests

from click import UsageError

from zen_knit.publish.support import cred

def get_workers(profile):
   
    access_key, secret_key , publish_url = cred.pull_cred(profile)

    data = {}
    data['access_key'] = access_key
    data['secret_key'] = secret_key

    r = requests.post( f"{publish_url}api/external/get_worker/", data=data)
    if r.status_code != 200:
        raise UsageError(r.text)
    else:
        response = r.json()

        if response["data"] is None :
            print("no worker name present")
            return 0

        workerNames = ', '.join( d["name"] for d in response["data"])
        print(f" workers are {workerNames}")
        return 0


def get_worker_groups(profile):
    access_key, secret_key , publish_url = cred.pull_cred(profile)

    data = {}
    data['access_key'] = access_key
    data['secret_key'] = secret_key

    r = requests.post( f"{publish_url}api/external/get_worker_group/", data=data)
    if r.status_code != 200:
        raise UsageError(r.text)
    else:
        response = r.json()
        if response["data"] is None :
            print("no worker group name present")

            return 0
        workerGroupNames = ', '.join( d["name"] for d in response["data"])
        print(f" worker group are {workerGroupNames}")
        return 0



import re
from zen_knit.data_types import GlobalOption, ReadData, ChunkOption, Chunk, ParseData, BOOLENCHUNCKOPTIONS
import oyaml
import datetime

def merge(a, b, path=None):
    "merges b into a"
    if path is None: path = []
    for key in b:
        if key in a:
            if isinstance(a[key], dict) and isinstance(b[key], dict):
                merge(a[key], b[key], path + [str(key)])
            elif a[key] == b[key]:
                pass # same leaf value
            else:
                print(f"over writing deafult value for {key}")
                a[key] = b[key]

        else:
            a[key] = b[key]
    return a


def option_parser(raw: str) -> ChunkOption:
    options = raw.split("{")[1].split("}")[0].replace(" ", "").split(",")
    option_dict = {}
    for option in options:
        if "=" not in option :
            continue
        passed_option, value = option.split("=")
        if passed_option in BOOLENCHUNCKOPTIONS:
            value = (value == "true") or (value == "True") or (value == "t") or (value == "T")
        option_dict[passed_option] = value
    selected_options = ChunkOption(**option_dict)
    return selected_options


class BaseParser:
    
    def __init__(self, data:ReadData) -> None:
        self.raw_data = data
        self.parsed_data = None
        self.starter_line = "^```\{.*?\}\s*$"
        self.ender_line = "^```$"
        self.title_info = {}
        self._parse_data()

    def _parse_title(self, title_info):
        old_data = self.raw_data.global_options.dict()
        new_data = oyaml.safe_load(title_info)
        final_data = merge(old_data, new_data)
        if  "CURRENT_DATE" in final_data["date"]:
            date1 =  datetime.datetime.now().strftime("%d %B, %Y")
        elif "datetime" in final_data["date"]:
            date1 = eval(final_data["date"])
        final_data["date"] = date1

        self.raw_data.global_options = GlobalOption(**final_data)


    def _parse_data(self):
        started = False
        current_strings = []
        temp_string = ''
        type_ = None
        option = option_parser("{}")
        if re.match("^---$", self.raw_data.data[:3]):
            _, title, raw_data = re.split("---\n", self.raw_data.data)
            self._parse_title(title)
        else:
            raw_data = self.raw_data.data
        for _, raw in enumerate(raw_data.splitlines()):
            if re.match(self.starter_line, raw):
                if started:
                    raise Exception("code is already started")
                else:
                    current_strings.append(Chunk(type= 'text' if type_ is None else type_,
                                                 string_= temp_string, 
                                                 options= option))
                    temp_string = ''
                    started = True
                    if ('{sql' in raw.lower()):
                        type_ = 'sql'
                    else:
                        type_ = 'code'  
                    option = option_parser(raw)

            else:
                if re.match(self.ender_line, raw):
                    if started:
                        current_strings.append(Chunk(type= 'text' if type_ is None else type_,
                                                 string_= temp_string, 
                                                 options= option))
                        temp_string = ''
                        started = False
                        type_ = None
                        option = option_parser("{}")
                        

                    else:
                        raise Exception("code is not started")
                else:
                    if temp_string != "":
                        temp_string = temp_string + "\n" + raw
                    else:
                        temp_string = raw
        
        current_strings.append(Chunk(type= 'text' if type_ is None else type_,
                                                 string_= temp_string, 
                                                 options= option))

        self.parsed_data = ParseData(global_options=self.raw_data.global_options, chunks=current_strings)



import os

import oyaml as yaml

from click import UsageError
import pathlib

def cred_path()-> str:
    base_path = os.path.expanduser("~")
    setting_path = base_path + "/.zen/setting.yml"
    return setting_path

def cred_file_present():
    setting_path = cred_path()

    if not os.path.isfile(setting_path):
        raise UsageError("zen report data is not present please use zen create command")
    
    return setting_path
        
def cred_folder():
    base_path = os.path.expanduser("~")
    folder_path = f"{base_path}/.zen"
    if not os.path.isdir(folder_path):
        os.mkdir(folder_path)
    return folder_path


def pull_cred(profile):
    cred_folder()    
    
    setting_path = cred_file_present()
    with open(setting_path, "r") as f:
        data = yaml.safe_load(f)
    
    if profile not in data.keys():
        raise UsageError(f"{profile} is not present in file")
    
    publish_url = data[profile]['url']
    if publish_url[-1] != "/":
        publish_url = publish_url + "/"
    access_key = data[profile]["key"]
    secret_key = data[profile]["secret"]
    
    return access_key, secret_key, publish_url


def create(key, secret, url, profile):
    folder_path  = cred_folder()    
    setting_path = cred_path()

    p = pathlib.Path(folder_path)
    p.mkdir(parents=True, exist_ok=True)
    
    if not os.path.exists(setting_path):
        my_dict = {
            profile: {
                "key": key,
                "secret": secret,
                "url": url
            }
        }
    else:
      
        with open(setting_path, "r") as f:
            my_dict = yaml.safe_load(f)

        my_dict[profile] = {
            "key": key,
            "secret": secret,
            "url": url
        }
    with open(setting_path, "w") as f:
        yaml.safe_dump(my_dict, f)

    return "created value"


def update(key, secret, profile):
    cred_folder()    
    setting_path = cred_file_present()

    with open(setting_path, "r") as f:
        data = yaml.safe_load(f)
    
    if profile not in data.keys():
        raise UsageError(f"{profile} is not present in file")

    data[profile]['key'] = key
    data[profile]['secret'] = secret

    with open(setting_path, "w") as f:
        yaml.dump(data, f)

    return "updated value"


import os
import shutil
import copy
import time
import webbrowser
import oyaml as yaml
import requests
from click import UsageError

from zen_knit.publish.support import cred

def publish(path, fresh, browser, profile):
    old_path = os.getcwd()
    if path != old_path:
        os.chdir(path)
    
    current = path
    files = os.listdir(current)
    present_config = any([True for file_ in files if 'config.yml' == file_])

    if not present_config:
        raise UsageError("config.yml file is not present")
  
    upload_file = {}
    for file_ in files:
        if file_ == 'config.yml':
            with open("config.yml", "r") as f:
                data = yaml.safe_load(f)
            break_list = ['title', 'type', 'language']
            for b in break_list:
                if b not in data.keys():
                    raise UsageError(f"{b.capitalize()} is not provided in config")
            if fresh:
                try:
                    del data['all_ids'][profile]
                except:
                    pass
                with open("config.yml", "w") as f:
                    yaml.dump(data, f)
            config = copy.deepcopy(data)
            
    if data.get("all_ids") is not None:
        if data["all_ids"].get(profile) is not None:
            data["id"] = data["all_ids"].get(profile)
    
    type = config.get('type')

    shutil.make_archive(".", "zip", ".")
    upload_file['zip_file'] = open(f"{path}.zip", "rb")

    access_key, secret_key , publish_url = cred.pull_cred(profile)
    data['access_key'] = access_key
    data['secret_key'] = secret_key

    print(f"publishing document at {publish_url}publish/report/")
    r = requests.post( f"{publish_url}publish/report/", files=upload_file, data=data)
    os.remove(f"{path}.zip")
    if r.status_code != 200:
        if r.status_code == 402:
            print("You have used all reports, need to updagrade Zen Reportz")
        raise UsageError(r.text)
    else:
        response = r.json()["data"]
        time.sleep(10)
        report_id = str(response["id"])
        config['all_ids'] = {
            profile: report_id
        }
        with open("config.yml", "w") as f:
            yaml.dump(config, f)

        if len(response["workers"])> 0 :
            workerNames = ', '.join(response["workers"])
            print(f"following worker(s) are not present, please talk your admin {workerNames}")

        if len(response["worker_groups"])> 0 :
            worker_group = ', '.join(response["worker_groups"])
            print(f"following worker group(s) are not present, please talk your admin {worker_group}")

        url_to_open = publish_url + f'report-application/{report_id}?type={type}&open=true'

        if browser:
            webbrowser.open(url_to_open)

        os.chdir(old_path)

    return "publish report"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zen_knit',
 'zen_knit.data_types',
 'zen_knit.executor',
 'zen_knit.formattor',
 'zen_knit.formattor.html_support',
 'zen_knit.organizer',
 'zen_knit.parser',
 'zen_knit.publish',
 'zen_knit.publish.core',
 'zen_knit.publish.support',
 'zen_knit.reader']

package_data = \
{'': ['*']}

install_requires = \
['Markdown>=3.3.6,<4.0.0',
 'SQLAlchemy>=1.4.29,<2.0.0',
 'click>=8.0.3,<9.0.0',
 'ipykernel>=6.0.0',
 'ipython>=7.0',
 'joblib>=1.1.0,<2.0.0',
 'jupyter-client>=7.1.0,<8.0.0',
 'nbconvert>=6.0.0',
 'nbformat>=5.1.3,<6.0.0',
 'oyaml>=1.0,<2.0',
 'pydantic>=1.8.2,<2.0.0',
 'requests>=2.27.1,<3.0.0']

entry_points = \
{'console_scripts': ['knit = zen_knit.cli:knit',
                     'zen = zen_knit.publish_cli:cli']}

setup_kwargs = {
    'name': 'zen-knit',
    'version': '0.2.4',
    'description': 'Zen-Knit is a formal (PDF), informal (HTML) report generator for data analyst and data scientist who wants to use python. Rmarkdown Alternative for Python, It also allow you to publish reports to Zen Reportz Enterprise or community edition',
    'long_description': '# About Zen-Knit:\nZen-Knit is a formal (PDF), informal (HTML) report generator for data analyst and data scientist who wants to use python. RMarkdown alternative.\nZen-Knit is good for creating reports, tutorials with embedded python. RMarkdown alternative. Python Markdown Support. It also allows you to publish reports to analytics-reports.zenreportz.com (comming soon) or private zenreportz servers \n\n\n[![Download Count](https://static.pepy.tech/personalized-badge/zen-knit?period=total&units=international_system&left_color=grey&right_color=green&left_text=Downloads)](https://pepy.tech/project/zen-knit)\n[![python](https://img.shields.io/pypi/pyversions/zen-knit.svg?color=green)](https://img.shields.io/pypi/pyversions/zen-knit.svg?color=green)\n[![license](https://img.shields.io/github/license/Zen-Reportz/zen_knit?color=green)](https://img.shields.io/github/license/Zen-Reportz/zen_knit?color=green)\n[![version](https://img.shields.io/pypi/v/zen-knit?color=green&label=pypi%20package)](https://img.shields.io/pypi/v/zen-knit?color=green&label=pypi%20package)\n\n\n# VS Code Plugin:\nDownload VS Plugin from [MarketPlace](https://marketplace.visualstudio.com/items?itemName=ZenReportz.vscode-zen-knit)\n\n\n# Features:\n* Python 3.7+ compatibility\n* Support for IPython magics and rich output.\n* **Execute python code** in the chunks and **capture** input and output to a report.\n* **Use hidden code chunks,** i.e. code is executed, but not printed in the output file.\n* Capture matplotlib graphics.\n* Evaluate inline code in documentation chunks marked using ```{ }``` \n* Publish reports from Python scripts. Similar to R markdown.\n* Interactive Plots using plotly\n* integrate it in your process. It will fit your need rather than you need to adjust for tool.\n* custom CSS support (HTTP(s) and local file)\n* direct sql support \n* chaching executed code for faster report devlopement \n* printing index of chunk or chunk name in console\n  \n# Examples:\nAll example are available [HERE](https://github.com/Zen-Reportz/zen_knit/tree/main/doc/example)\n\n\n## PDF example\n![PDF Code](./doc/example/screenshots/pdf_code.png)\n![PDF Output](./doc/example/screenshots/pdf_output.png)\n\n## PDF example with SQL\n![PDF SQL Code](./doc/example/screenshots/pdf_sql_code.png) \n![PDF SQL Output](./doc/example/screenshots/pdf_sql_output.png) \n\n## HTML example\n![HTML Code](./doc/example/screenshots/html_code.png) \n![HTML Ouput ](./doc/example/screenshots/html_output.png)\n![HTML output 2](./doc/example/screenshots/html_output_2.png)\n\n## HTML example with custom CSS\n![HTML CDN CSS](./doc/example/screenshots/html_cdn_css_code.png) \n![HTML Custom CSS](./doc/example/screenshots/html_custom_css_code.png)\n\n## HTML example with SQL\n![HTML SQL](./doc/example/screenshots/html_sql_code.png) \n![HTML SQL output](./doc/example/screenshots/html_sql_output.png) \n\n\n# Install\n\nFrom PyPi:\n\n  <code> pip install --upgrade zen-knit </code>\n\nor download the source and run::\n\n  <code> python setup.py install </code>\n\n\n## Other Dependency\n\n<code> install pandoc from : https://github.com/jgm/pandoc/releases </code>\n\n<code> install texlive for debian: sudo apt install texlive-full </code>\n\n<code> install texlive for window: https://www.tug.org/texlive/acquire-netinstall.html </code>\n\n<code> install texlive for mac: https://tug.org/texlive/quickinstall.html </code>\n\n\n## License information\n\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n"Software"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n\n\n## How to Use it\n\n  <code> pip install zen-knit  </code>\n\n  <code> knit -f doc/example/html_example.pyz  -ofd doc/example/output/  </code>\n  \n  <code> knit -f doc/example/pdf_example.pyz  -ofd doc/example/output/  </code>\n\n  <code>  python doc/example/demo.py  </code>\n  \n\n## Arguments \n    ---\n    title: Zen Markdown Demo\n    author: Dr. P. B. Patel\n    date: CURRENT_DATE\n    cache: true\n    output: \n        format: html\n        html: \n            css: skleton\n    ---\n\nAbove code will map on GlobalOption class in in following\n\n    class Input(BaseModel):\n        dir: Optional[str]\n        file_name: Optional[str]\n        matplot: bool = True\n    \n    class latexOuput(BaseModel):\n        header: Optional[str] \n        page_size: Optional[str] = \'a4paper\'\n        geometry_parameter: Optional[str] = "text={16.5cm,25.2cm},centering"  #Newely added parameters\n\n    class htmlOutput(BaseModel):\n        css: str = "bootstrap"\n\n    class Output(BaseModel):\n        fig_dir: str = "figures"\n        format: Optional[str]\n        file_name: Optional[str]\n        dir: Optional[str]\n        latex: Optional[latexOuput]\n        html: Optional[htmlOutput]\n\n    class GlobalOption(BaseModel):\n        title: str\n        author: Optional[str]\n        date: Optional[str]\n        kernal: str = "python3"\n        log_level: str = "debug"\n        cache: Optional[bool] = False\n        output: Output\n        input: Input\n\n        @validator(\'log_level\')\n        def fix_option_for_log(cls, v:str):\n            if v.lower() not in (\'notset\', "debug", \'info\', \'warning\', \'error\', \'critical\'):\n                raise ValueError(\'must contain a space\')\n            return v.title()\n\n\n# Zen Publish:\nAbility to publish programmable, formal, informal reports to Private or Public instance of zen reportz.\nLearn more at [Here](https://zenreportz.com?utm=github)\n\nLearn more about how to publish to private or public instance of Zen Reportz [Here](https://zenreportz.com/how-to-publish-to-zen-reportz?utm=github)\n\n# analytics-reports.zenreportz.com features\n* Static Reports like HTML, PDF\n* Any one access reports \n* Free to use \n* Unlimite Publish\n* Republish report same place again\n',
    'author': 'Zen',
    'author_email': 'zenreportz@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Zen-Reportz/zen_knit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import click
import os

from zen_knit.publish.support import cred
from zen_knit.publish.core import publish as my_publish, worker




@click.group()
def cli():
    pass


@click.command()
@click.option('--key', prompt='what is your key?', help='What is your access key?')
@click.option('--secret', prompt='what is your secret key?', help='What is your secret key?')
@click.option('--url', prompt='what is zen report url?', help='what is zen report url?')
@click.option('--profile', help='which profile do you want to use?', default='default')
def create(key, secret, url, profile):
    return cred.create(key, secret, url, profile)


@click.command()
@click.option('--key', help='What is your access key?')
@click.option('--secret', help='What is your secret key?')
@click.option('--url', help='what is zen report url?')
@click.option('--profile', help='which profile do you want to use?', default='default')
def create_w(key, secret, url, profile):
    return cred.create(key, secret, url, profile)


@click.command()
@click.option('--key', prompt='what is your access key?', help='What is your access key?')
@click.option('--secret', prompt='what is your secret key?', help='What is your secret key?')
@click.option('--profile', help='which profile do you want to use?', default='default')
def update(key, secret, profile):
    return cred.update(key, secret, profile)


@click.command()
@click.option('--key', help='What is your access key?')
@click.option('--secret', help='What is your secret key?')
@click.option('--profile', help='which profile do you want to use?', default='default')
def update_w(key, secret, profile):
    return cred.update(key, secret, profile)

@click.command()
@click.option('--path', prompt='path of report or dashboard?', default=os.getcwd(), help="folder to look into")
@click.option('--fresh', '-f', prompt='do you want to publish as fresh report',  is_flag=True,  help="run as fresh report")
@click.option('--browser','-b',prompt='do you want to open report in browser', is_flag=True, help="Want to open window")
@click.option('--profile', prompt='which profile do you want to use?', help='which profile do you want to use?', default='default')
def publish(path, fresh, browser, profile):
    return my_publish.publish(path, fresh, browser, profile)


@click.command()
@click.option('--path', default=os.getcwd(), help="folder to look into")
@click.option('--fresh', '-f', is_flag=True,  help="run as fresh report")
@click.option('--browser','-b', is_flag=True, help="Want to open window")
@click.option('--profile', help='which profile do you want to use?', default='default')
def publish_w(path, fresh, browser, profile):
    return my_publish.publish_w(path, fresh, browser, profile)

@click.command()
@click.option('--profile', help='which profile do you want to use?', default='default')
def get_workers(profile):
    return worker.get_workers(profile)

@click.command()
@click.option('--profile', help='which profile do you want to use?', default='default')
def get_worker_group(profile):
    return worker.get_worker_groups(profile)



cli.add_command(update)
cli.add_command(update_w)

cli.add_command(create)
cli.add_command(create_w)

cli.add_command(publish)
cli.add_command(publish_w)

cli.add_command(get_workers)
cli.add_command(get_worker_group)
